@@ -319,6 +319,17 @@ class TestOneOf(unittest.TestCase):
         e = invalid_exc(validator, None, None)
         self.assertEqual(e.msg.interpolate(), '"None" is not one of 1, 2')
 
+class TestSchemaType(unittest.TestCase):
+    def _makeOne(self, *arg, **kw):
+        from colander import SchemaType
+        return SchemaType(*arg, **kw)
+
+    def test_flatten(self):
+        node = DummySchemaNode(None, name='node')
+        typ = self._makeOne()
+        result = typ.flatten(node, 'appstruct')
+        self.assertEqual(result, {'node':'appstruct'})
+
 
 class TestMapping(unittest.TestCase):
     def _makeOne(self, *arg, **kw):
@@ -439,6 +450,19 @@ class TestMapping(unittest.TestCase):
         result = typ.serialize(node, null)
         self.assertEqual(result, {'a':null})
 
+    def test_flatten(self):
+        node = DummySchemaNode(None, name='node')
+        int1 = DummyType()
+        int2 = DummyType()
+        node.children = [
+            DummySchemaNode(int1, name='a'),
+            DummySchemaNode(int2, name='b'),
+            ]
+        typ = self._makeOne()
+        result = typ.flatten(node, {'a':1, 'b':2})
+        self.assertEqual(result,
+                         {'node': {'a': 1, 'b': 2}, 'node.appstruct': 2})
+
 class TestTuple(unittest.TestCase):
     def _makeOne(self):
         from colander import Tuple
@@ -550,6 +574,18 @@ class TestTuple(unittest.TestCase):
         self.assertEqual(e.msg, None)
         self.assertEqual(len(e.children), 2)
 
+    def test_flatten(self):
+        node = DummySchemaNode(None, name='node')
+        int1 = DummyType()
+        int2 = DummyType()
+        node.children = [
+            DummySchemaNode(int1, name='a'),
+            DummySchemaNode(int2, name='b'),
+            ]
+        typ = self._makeOne()
+        result = typ.flatten(node, (1, 2))
+        self.assertEqual(result, {'node': (1, 2), 'node.appstruct': 2})
+
 class TestSequence(unittest.TestCase):
     def _makeOne(self, **kw):
         from colander import Sequence
@@ -646,6 +682,24 @@ class TestSequence(unittest.TestCase):
         self.assertEqual(e.msg, None)
         self.assertEqual(len(e.children), 2)
 
+    def test_flatten(self):
+        node = DummySchemaNode(None, name='node')
+        int1 = DummyType()
+        int2 = DummyType()
+        node.children = [
+            DummySchemaNode(int1, name='a'),
+            DummySchemaNode(int2, name='b'),
+            ]
+        typ = self._makeOne()
+        result = typ.flatten(node, [1, 2])
+        self.assertEqual(
+            result,
+            {'node': [1, 2],
+             'node.0': 1,
+             'node.0.appstruct': 1,
+             'node.1.appstruct': 2,
+             'node.1': 2})
+
 class TestString(unittest.TestCase):
     def _makeOne(self, encoding=None):
         from colander import String
@@ -1310,8 +1364,9 @@ class TestSchemaNode(unittest.TestCase):
         node = self._makeOne(None, 0, widget='abc')
         self.assertEqual(node.widget, 'abc')
 
-    def test_ctor_with_bad_kwargs(self):
-        self.assertRaises(TypeError, self._makeOne, None, 0, foo=1)
+    def test_ctor_with_unknown_kwarg(self):
+        node = self._makeOne(None, 0, foo=1)
+        self.assertEqual(node.foo, 1)
 
     def test_required_true(self):
         node = self._makeOne(None)
@@ -1321,6 +1376,11 @@ class TestSchemaNode(unittest.TestCase):
         node = self._makeOne(None, missing=1)
         self.assertEqual(node.required, False)
 
+    def test_required_deferred(self):
+        from colander import deferred
+        node = self._makeOne(None, missing=deferred('123'))
+        self.assertEqual(node.required, True)
+
     def test_deserialize_no_validator(self):
         typ = DummyType()
         node = self._makeOne(typ)
@@ -1361,6 +1421,15 @@ class TestSchemaNode(unittest.TestCase):
         node.missing = null
         self.assertEqual(node.deserialize(null), null)
 
+    def test_deserialize_appstruct_deferred(self):
+        from colander import null
+        from colander import deferred
+        from colander import Invalid
+        typ = DummyType()
+        node = self._makeOne(typ)
+        node.missing = deferred('123')
+        self.assertRaises(Invalid, node.deserialize, null)
+
     def test_serialize(self):
         typ = DummyType()
         node = self._makeOne(typ)
@@ -1388,6 +1457,14 @@ class TestSchemaNode(unittest.TestCase):
         node.default = 'abc'
         self.assertEqual(node.serialize(), 'abc')
 
+    def test_serialize_default_deferred(self):
+        from colander import deferred
+        from colander import null
+        typ = DummyType()
+        node = self._makeOne(typ)
+        node.default = deferred('abc')
+        self.assertEqual(node.serialize(), null)
+
     def test_add(self):
         node = self._makeOne(None)
         node.add(1)
@@ -1594,20 +1671,73 @@ class TestFunctional(object):
         self.assertEqual(result['seq2'],
                          [{'key':1, 'key2':2}, {'key':3, 'key2':4}])
         self.assertEqual(result['tup'], (1, 's'))
+
+    def test_flatten_ok(self):
+        import colander
+        appstruct = {
+            'int':10,
+            'ob':colander.tests,
+            'seq':[(1, 's'),(2, 's'), (3, 's'), (4, 's')],
+            'seq2':[{'key':1, 'key2':2}, {'key':3, 'key2':4}],
+            'tup':(1, 's'),
+            }
+        schema = self._makeSchema()
+        result = schema.flatten(appstruct)
+
+        expected = {
+            'schema.seq.2.tup.tupstring': 's',
+            'schema.seq2.0.mapping.key2': 2,
+            'schema.seq.0': (1, 's'),
+            'schema.seq.1': (2, 's'),
+            'schema.seq.2': (3, 's'),
+            'schema.seq.3': (4, 's'),
+            'schema.seq': [(1, 's'), (2, 's'), (3, 's'), (4, 's')],
+            'schema.ob': colander.tests,
+            'schema.seq2.1.mapping.key2': 4,
+            'schema.seq.0.tup': (1, 's'),
+            'schema.seq.1.tup': (2, 's'),
+            'schema.seq2.0.mapping': {'key2': 2, 'key': 1},
+            'schema.seq2.1.mapping': {'key2': 4, 'key': 3},
+            'schema.seq.1.tup.tupstring': 's',
+            'schema.seq2.0.mapping.key': 1,
+            'schema.seq.1.tup.tupint': 2,
+            'schema.tup': (1, 's'),
+            'schema.seq.3.tup': (4, 's'),
+            'schema.seq.0.tup.tupstring': 's',
+            'schema.seq.2.tup': (3, 's'),
+            'schema.seq.3.tup.tupstring': 's',
+            'schema.seq.3.tup.tupint': 4,
+            'schema.seq2.1.mapping.key': 3,
+            'schema.int': 10,
+            'schema.seq2.0': {'key2': 2, 'key': 1},
+            'schema.seq.0.tup.tupint': 1,
+            'schema.tup.tupint': 1,
+            'schema.tup.tupstring': 's',
+            'schema.seq.2.tup.tupint': 3,
+            'schema.seq2': [{'key2': 2, 'key': 1}, {'key2': 4, 'key': 3}],
+            'schema.seq2.1': {'key2': 4, 'key': 3},
+            'schema': {'int': 10,
+                       'seq2': [{'key2': 2, 'key': 1}, {'key2': 4, 'key': 3}],
+                       'tup': (1, 's'),
+                       'ob':colander.tests,
+                       'seq': [(1, 's'), (2, 's'), (3, 's'), (4, 's')]}}
+
+        for k, v in result.items():
+            self.assertEqual(v, expected[k])
         
     def test_invalid_asdict(self):
         expected = {
-            'int': '20 is greater than maximum value 10',
-            'ob': 'The dotted name "no.way.this.exists" cannot be imported',
-            'seq.0.0': '"q" is not a number',
-            'seq.1.0': '"w" is not a number',
-            'seq.2.0': '"e" is not a number',
-            'seq.3.0': '"r" is not a number',
-            'seq2.0.key': '"t" is not a number',
-            'seq2.0.key2': '"y" is not a number',
-            'seq2.1.key': '"u" is not a number',
-            'seq2.1.key2': '"i" is not a number',
-            'tup.0': '"s" is not a number'}
+            'schema.int': '20 is greater than maximum value 10',
+            'schema.ob': 'The dotted name "no.way.this.exists" cannot be imported',
+            'schema.seq.0.0': '"q" is not a number',
+            'schema.seq.1.0': '"w" is not a number',
+            'schema.seq.2.0': '"e" is not a number',
+            'schema.seq.3.0': '"r" is not a number',
+            'schema.seq2.0.key': '"t" is not a number',
+            'schema.seq2.0.key2': '"y" is not a number',
+            'schema.seq2.1.key': '"u" is not a number',
+            'schema.seq2.1.key2': '"i" is not a number',
+            'schema.tup.0': '"s" is not a number'}
         data = {
             'int':'20',
             'ob':'no.way.this.exists',
@@ -1678,7 +1808,8 @@ class TestImperative(unittest.TestCase, TestFunctional):
             ob,
             tup,
             seq,
-            seq2)
+            seq2,
+            name='schema')
 
         return schema
 
@@ -1698,7 +1829,7 @@ class TestDeclarative(unittest.TestCase, TestFunctional):
             key2 = colander.SchemaNode(colander.Int())
 
         class SequenceOne(colander.SequenceSchema):
-            tuple = TupleSchema()
+            tup = TupleSchema()
 
         class SequenceTwo(colander.SequenceSchema):
             mapping = MappingSchema()
@@ -1711,7 +1842,7 @@ class TestDeclarative(unittest.TestCase, TestFunctional):
             tup = TupleSchema()
             seq2 = SequenceTwo()
 
-        schema = MainSchema()
+        schema = MainSchema(name='schema')
         return schema
 
 class Test_null(unittest.TestCase):
@@ -1769,3 +1900,7 @@ class DummyType(object):
     def deserialize(self, node, value):
         return value
 
+    def flatten(self, node, appstruct, prefix=''):
+        key = prefix + 'appstruct'
+        return {key:appstruct}
+
